// Copyright (c) 1999, 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: llscmd.java,v 1.3 2000/01/18 04:54:23 maurice Exp $


package cmd;
import java.util.*;
import java.io.*;

public class llscmd extends Cmd
{
	static int numCols = 80;
	static int tabstop = 12;
	private StringBuffer cwd;
	private cmd.cmdio inStream;
	public llscmd(cmd.cmdio inStream, StringBuffer cwd)
	{
		this.cwd = cwd;
		this.inStream = inStream;
	}
	public String help(String cmd)
	{
		return(cmd + " [directory_name]");
	}
	public void exec(StringTokenizer st) throws java.io.IOException
	{
		File newcwd;
		if (st.hasMoreTokens())
		{
			String dir = inStream.readLine(st, "(local-directory): ");
			// check if they typed in an absolute path
			newcwd = new File(dir);
			if (!dir.equals(newcwd.getAbsolutePath()))
			{
				// if the path is not absolute build a new path
				newcwd = new File(cwd.toString(), dir);
			}
			if (!newcwd.isDirectory())
			{
				System.out.println("LCD failed " + dir + " not a directory");
				return;
			}
		}
		else
			newcwd = new File(cwd.toString());
		// read directory
		String[] filenames = newcwd.list();
		int i;
		int col = 0;
		for (i = 0; i < filenames.length; i++)
		{
			if (col + tabstop >= numCols)
			{
				System.out.println();
				col = 0;
			}
			else
			{
				while (col % tabstop != 0)
				{
					System.out.print(" ");
					col++;
				}
			}
			col = col + filenames[i].length() + 1;
			if (col >= numCols)
			{
				System.out.println();
				col = filenames[i].length() + 1;
			}
			System.out.print(filenames[i] + " ");
		}
		System.out.println();
	}
}
