// Copyright (c) 1999, 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: mputcmd.java,v 1.3 2000/01/18 04:54:24 maurice Exp $


package cmd;
import java.util.*;
import java.io.*;
import ftp.ftp;

public class mputcmd extends putcmd
{
	public mputcmd(ftp ftp, StringBuffer cwd, cmd.cmdio inStream)
	{
		super(ftp, cwd, inStream);
	}
	public String help(String cmd)
	{
		return(cmd + " [file_name ...]");
	}
	public static boolean strmtch(String pattern, String string)
	{
		char [] pat = pattern.toCharArray();
		char [] str = string.toCharArray();
		int pp = 0;
		int sp = 0;
		int i;
		boolean negate = false;
		boolean charMatch = false;
		char highChar;
		char lowChar;
		for (;;)
		{
			// termination - end of both pattern and string
			if ((pp == pat.length) && (sp == str.length))
				return(true);
			// termination - end of pattern
			if (pp == pat.length)
				return(false);
			// handle match all (including nothing) case
			switch (pat[pp])
			{
				case '*':
					// try to match all remainders of str
					for (i = sp ; i <= str.length; i++)
					{
						if (strmtch(pattern.substring(pp+1), string.substring(i)) )
						{
							return(true);
						}
					}
					return(false);
			}
			// termination - end of string
			if (sp == str.length)
				return(false);
			// determine items to match
			switch (pat[pp])
			{
				case '\\':
					// quote and match a character
					pp += 1;
					if (pat[pp] != str[sp])
					{
						return(false);
					}
					pp += 1;
					break;
				case '?':
					// match any character
					pp += 1;
					break;
				case '[':
					// match range of characters
					charMatch = false;
					// check for pattern negation
					pp += 1;
					if (pat[pp] == '!')
					{
						negate = true;
						pp += 1;
					}
					else
					{
						negate = false;
					}
					while (pat[pp] != ']')
					{
						if (pat[pp + 1] == '-')
						{
							lowChar = pat[pp];
							highChar = pat[pp+2];
							if ((lowChar <= str[sp]) && (str[sp] <= highChar))
							{
								charMatch = true;
							}
							pp += 2;
						}
						else if (pat[pp] == str[sp])
						{
							charMatch = true;
						}
						pp += 1;
					}
					pp += 1;
					if (negate && charMatch)
						return(false);
					if (!negate && !charMatch)
						return(false);
					break;
				default:
					// match an ordinary character
					if (pat[pp] != str[sp])
                                        {
                                                return(false);
                                        }
					pp += 1;
					break;;
			}
			sp += 1;
		}
	}
	public StringBuffer listFiles(String pattern)
	{
		int i;
		StringBuffer result = new StringBuffer();
		File newcwd = new File(cwd.toString());
		String[] filenames = newcwd.list();
		for (i = 0; i < filenames.length; i++)
		{
			if (strmtch(pattern, filenames[i]))
				result.append(filenames[i] + "\n");
		}
		return(result);
	}
	public void exec(StringTokenizer st) throws java.io.IOException
	{
		StringBuffer filenames;
		String pattern  = inStream.readLine(st, "(local-files): ");
		filenames = listFiles(pattern);
		while (st.hasMoreTokens())
		{
			pattern  = st.nextToken();
			filenames.append(listFiles(pattern));
		}
		putFiles(filenames);
	}
	private void putFiles(StringBuffer filenames) throws java.io.IOException
	{
		StringTokenizer tok = new StringTokenizer(filenames.toString(), "\n");
		while(tok.hasMoreTokens())
		{
			putFile(tok.nextToken());
		}
	}
}
