// Copyright (c) 1999, 2000
//      Maurice Castro (maurice@serc.rmit.edu.au)
//      All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 
// This software is provided by the original author and contributors ``as
// is'' and any express or implied warranties, including, but not limited
// to, the implied warranties of merchantability and fitness for a
// particular purpose are disclaimed.  In no event shall the authors or
// contributors be liable for any direct, indirect, incidental, special,
// exemplary, or consequential damages (including, but not limited to,
// procurement of substitute goods or services; loss of use, data, or
// profits; or business interruption) however caused and on any theory of
// liability, whether in contract, strict liability, or tort (including
// negligence or otherwise) arising in any way out of the use of this
// software, even if advised of the possibility of such damage.
//
// $Id: pjftp.java,v 1.6 2000/01/21 07:45:54 maurice Exp $


package main;

import java.io.*;
import java.util.*;
import ftp.ftp;
import lclhst;

public class pjftp 
{
	private static String banner = new String(
		"\n" +
		"              pjftp: Psion Java FTP\n\n" +
		"      Small Ftp Client for Psion Series 5mx\n" +
		" Author: Maurice Castro (maurice@serc.rmit.edu.au)\n" +
		"        Version: MAJORVERNUM.MINORVERNUM   Date: DATETODAY\n" +
		"\n"
		);
	private static String prompt = new String("ftp> ");
	private static Hashtable cmdTab;
	private static ftp ftp;
	private static cmd.cmdio inStream = new cmd.cmdio(System.in);
	private static StringBuffer cwd;
	private static lclhst lhost;

	public static void main(String [] args)
	{
		System.out.println(banner);
		try
		{
 			cwd = new StringBuffer((new File("/")).getCanonicalPath());
		}
		catch (java.io.IOException e)
		{
			System.out.println("Can not get cwd");
			return;
		}
		lhost = new lclhst();
		ftp = new ftp();
		ftp.passive();
		ftp.ascii();
		cmdTab = PopulateCmdTab();
		CommandLoop();
	}

// perform commands repeatedly
// fetch line, 
// grab command - first word, 
// search for command 
// call object implementing command with remaining arguments in tokenizer

	private static void CommandLoop() 
	{
		String inString;
		String command;
		StringTokenizer st;
		while (true) 
		{
			try 
			{
				inString = inStream.readLine(prompt);
			}
			catch (java.io.IOException e)
			{
				System.out.print("IO exception: ");
				System.out.println(e);
				return;
			}
			st = new StringTokenizer(inString);
			if (st.hasMoreTokens())
			{
				command = st.nextToken();
				try
				{
					((cmd.Cmd) cmdTab.get(command)).exec(st);
				}
				catch (java.lang.NullPointerException e)
				{
					System.out.println("Invalid command");
				}
				catch (java.io.IOException e)
				{
					System.out.print("IO Error: ");
					System.out.println(e.getMessage());
				}
				// quit is unique and needs to be handled
				// as an exception
				if (command.equals("quit"))
				{
					return;
				}
			}
		}
	}

// put command objects into command table object
// command objects take a tokenizer as an argument

	static private Hashtable PopulateCmdTab()
	{
		Hashtable cmdTab = new Hashtable();

		cmdTab.put("help", new cmd.helpcmd(cmdTab));
		cmdTab.put("?", new cmd.helpcmd(cmdTab));
		cmdTab.put("dir", new cmd.dircmd(ftp, inStream));
		cmdTab.put("open", new cmd.opencmd(ftp, inStream));
		cmdTab.put("ls", new cmd.lscmd(ftp, inStream));
		cmdTab.put("cd", new cmd.cdcmd(ftp, inStream));
		cmdTab.put("user", new cmd.usercmd(ftp, inStream));
		cmdTab.put("quit", new cmd.quitcmd());
		cmdTab.put("close", new cmd.closecmd(ftp));
		cmdTab.put("lcd", new cmd.lcdcmd(inStream, cwd));
		cmdTab.put("get", new cmd.getcmd(ftp, cwd, inStream));
		cmdTab.put("recv", new cmd.getcmd(ftp, cwd, inStream));
		cmdTab.put("ascii", new cmd.asciicmd(ftp));
		cmdTab.put("binary", new cmd.bincmd(ftp));
		cmdTab.put("put", new cmd.putcmd(ftp, cwd, inStream));
		cmdTab.put("send", new cmd.putcmd(ftp, cwd, inStream));
		cmdTab.put("lls", new cmd.llscmd(inStream, cwd));
		cmdTab.put("ldir", new cmd.llscmd(inStream, cwd));
		cmdTab.put("delete", new cmd.delcmd(ftp, inStream));
		cmdTab.put("mkdir", new cmd.mdcmd(ftp, inStream));
		cmdTab.put("rmdir", new cmd.rmdcmd(ftp, inStream));
		cmdTab.put("mget", new cmd.mgetcmd(ftp, cwd, inStream));
		cmdTab.put("mput", new cmd.mputcmd(ftp, cwd, inStream));
		if (lhost.available())
		{
			cmdTab.put("active", new cmd.activecmd(ftp));
			cmdTab.put("passive", new cmd.passivecmd(ftp));
		}
		return(cmdTab);
	}
}

